/**
 * Enables lazy compilation of built assets.
 *
 * @example
 * Enable:
 *
 * ```js
 * bud.lazy()
 * ```
 *
 * @example
 * Explicitly disable:
 *
 * ```js
 * bud.lazy(false)
 * ```
 *
 * @example
 * Explicitly enable:
 *
 * ```js
 * bud.lazy(true)
 * ```
 *
 * @example
 * Use with object options:
 *
 * ```js
 * bud.lazy({
 *   entries: false,
 *   imports: true,
 * })
 * ```
 */
export const lazy = function (value = true) {
    if (typeof value === `boolean`) {
        this.hooks.on(`build.experiments`, experiments => ({
            ...(experiments ?? {}),
            lazyCompilation: value,
        }));
        return this;
    }
    this.hooks.on(`build.experiments`, experiments => {
        if (typeof experiments?.lazyCompilation === `boolean` ||
            !experiments?.lazyCompilation)
            return { ...(experiments ?? {}), lazyCompilation: value };
        experiments.lazyCompilation = {
            ...experiments.lazyCompilation,
            ...value,
        };
        return experiments;
    });
    return this;
};
